from __future__ import annotations

from typing import Optional, Dict, Any, TYPE_CHECKING


from checkov.common.graph.checks_infra.enums import SolverType

if TYPE_CHECKING:
    from checkov.common.bridgecrew.severities import Severity
    from checkov.policies_3d.runner import CVECheckAttribute


class Base3dPolicyCheck:
    def __init__(self) -> None:
        self.id = ""
        self.bc_id = ""
        self.name = ""
        self.category = ""
        self.type: Optional[SolverType] = None
        self.guideline: Optional[str] = None
        self.severity: Optional[Severity] = None
        self.bc_category: Optional[str] = None
        self.iac: Dict[str, Any] = {}
        self.cve: Dict[CVECheckAttribute, Any] = {}



from abc import abstractmethod
from typing import Dict, Any

from checkov.policies_3d.checks_infra.base_check import Base3dPolicyCheck


class Base3dPolicyCheckParser:
    @abstractmethod
    def parse_raw_check(self, raw_check: Dict[str, Dict[str, Any]], **kwargs: Any) -> Base3dPolicyCheck:
        pass




from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class HostPort(BaseResourceCheck):

    def __init__(self):
        """
               https://kubernetes.io/docs/concepts/configuration/overview/

               Don’t specify a hostPort for a Pod unless it is absolutely necessary.
               When you bind a Pod to a hostPort, it limits the number of places the
               Pod can be scheduled, because each <hostIP, hostPort, protocol> combination
               must be unique.
               """
        name = "Do not specify hostPort unless absolutely necessary"
        id = "CKV_K8S_26"
        supported_resources = ["kubernetes_pod", "kubernetes_pod_v1",
                               "kubernetes_deployment", "kubernetes_deployment_v1"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if "spec" not in conf:
            self.evaluated_keys = [""]
            return CheckResult.FAILED

        spec = conf.get('spec')[0]
        evaluated_keys_path = "spec"

        if spec:

            template = spec.get("template")
            if template and isinstance(template, list):
                template = template[0]
                template_spec = template.get("spec")
                if template_spec and isinstance(template_spec, list):
                    spec = template_spec[0]
                    evaluated_keys_path = f'{evaluated_keys_path}/[0]/template/[0]/spec'

            containers = spec.get("container")
            if containers is None:
                return CheckResult.UNKNOWN
            for idx, container in enumerate(containers):
                if not isinstance(container, dict):
                    return CheckResult.UNKNOWN
                if container.get("port"):
                    for idy, port in enumerate(container["port"]):
                        if "host_port" in port:
                            self.evaluated_keys = [f"{evaluated_keys_path}/[0]/container/[{idx}]/port/[{idy}]/host_port"]
                            return CheckResult.FAILED
                return CheckResult.PASSED

        return CheckResult.FAILED


check = HostPort()



from __future__ import annotations

from dataclasses import dataclass, field

from checkov.common.bridgecrew.severities import Severities
from checkov.common.output.record import DEFAULT_SEVERITY

UNKNOWN_LICENSE = 'Unknown'


@dataclass
class SCADetails:
    package_types: dict[str, str] = field(default_factory=dict)


@dataclass
class ImageDetails(SCADetails):
    distro: str = ''
    distro_release: str = ''
    image_id: str = ''
    name: str | None = ''
    related_resource_id: str | None = ''


def is_raw_formatted(licenses: str) -> bool:
    return '","' in licenses


def format_licenses_to_string(licenses_lst: list[str]) -> str:
    if isinstance(licenses_lst, list):
        if len(licenses_lst) > 1:
            joined_str = '","'.join(licenses_lst)
            return f'"{joined_str}"'
        elif licenses_lst:
            return licenses_lst[0]
    return UNKNOWN_LICENSE


def format_string_to_licenses(licenses_str: str) -> list[str]:
    if licenses_str == UNKNOWN_LICENSE:
        return [licenses_str]
    elif licenses_str:
        # remove first and last quotes
        licenses_str = licenses_str[1:-1] if licenses_str.startswith('"') and licenses_str.endswith('"') else licenses_str
        license_lst = licenses_str.split('","')

        return license_lst
    else:
        return []


def compare_table_items_severity(table_item: dict[str, str]) -> int:
    severity = (table_item.get("severity") or DEFAULT_SEVERITY).upper()
    return Severities[severity].level



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list


class AccessControlGroupInboundRule(BaseResourceCheck):
    def __init__(self, check_id: str, port: int) -> None:
        name = f"Ensure no access control groups allow inbound from 0.0.0.0:0 to port {port}"
        id = check_id
        supported_resource = ('ncloud_access_control_group_rule',)

        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resource)
        self.port = port

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:

        if 'inbound' in conf :  # This means it's an ACG resource with inbound block(s)
            inbound_conf = conf['inbound']
            for inbound_rule in inbound_conf:
                for rule in force_list(inbound_rule):
                    if isinstance(rule, dict) and self.contains_violation(rule):
                        self.evaluated_keys = [
                            f'inbound/[{inbound_conf.index(inbound_rule)}]/port_range',
                            f'inbound/[{inbound_conf.index(inbound_rule)}]/ip_block',
                        ]

                        return CheckResult.FAILED

        return CheckResult.PASSED

    def contains_violation(self, conf: dict[str, list[Any]]) -> bool:
        if 'port_range' in conf:
            port_range = conf.get('port_range')

            if isinstance(port_range, list):
                port_range = port_range[0].split("-")

                from_port = int(port_range[0])
                to_port = int(port_range[1]) if len(port_range) > 1 else from_port

                if to_port == 0 and from_port == 0:
                    to_port = 65535

                if from_port is not None and to_port is not None and (from_port <= self.port <= to_port):
                    conf_cidr_blocks = conf.get('ip_block', [[]])
                    if conf_cidr_blocks:
                        conf_cidr_blocks = conf_cidr_blocks[0]
                    cidr_blocks = force_list(conf_cidr_blocks)

                    if '0.0.0.0/0' in cidr_blocks:
                        return True

        return False



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class GKEBasicAuth(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure GKE basic auth is disabled"
        id = "CKV_GCP_19"
        supported_resources = ('google_container_cluster',)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # since GKE 1.19 the usage of basic auth is deprecated and in the provider version 4+ removed
        master_auth = conf.get("master_auth")
        if master_auth and isinstance(master_auth, list):
            username = master_auth[0].get('username')
            password = master_auth[0].get('password')
            if username or password:
                # only if both are set to the empty string it is fine
                # https://registry.terraform.io/providers/hashicorp/google/3.90.1/docs/resources/container_cluster.html
                if username and password:
                    if username[0] == '' and password[0] == '':
                        return CheckResult.PASSED
                return CheckResult.FAILED

        return CheckResult.PASSED

    def get_evaluated_keys(self) -> list[str]:
        return ['master_auth/[0]/username', 'master_auth/[0]/password']


check = GKEBasicAuth()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class PasswordPolicyNumber(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure RAM password policy requires at least one number"
        id = "CKV_ALI_14"
        supported_resources = ['alicloud_ram_account_password_policy']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'require_numbers'


check = PasswordPolicyNumber()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult
from checkov.github.base_github_org_check import BaseOrganizationCheck


class GithubRequireStrictBasePermissionsRepository(BaseOrganizationCheck):
    def __init__(self) -> None:
        # https://docs.github.com/en/organizations/managing-user-access-to-your-organizations-repositories/setting-base-permissions-for-an-organization
        name = "Ensure strict base permissions are set for repositories"
        id = "CKV_GITHUB_27"
        super().__init__(id=id, name=name, missing_attribute_result=CheckResult.FAILED)

    def get_evaluated_keys(self) -> list[str]:
        return ["default_repository_permission"]

    def get_allowed_values(self) -> list[Any]:
        return ['read', None]


check = GithubRequireStrictBasePermissionsRepository()



from __future__ import annotations

from collections.abc import Iterable

from checkov.common.checks.base_check import BaseCheck
from checkov.common.models.enums import CheckCategories

from checkov.yaml_doc.registry import registry


class BaseYamlCheck(BaseCheck):
    def __init__(
        self, name: str,
        id: str,
        categories: Iterable[CheckCategories],
        supported_entities: Iterable[str],
        block_type: str,
        path: str | None = None,
    ) -> None:
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_entities,
            block_type=block_type,
        )
        self.path = path
        registry.register(self)
